/*
 *	$Id: stack.c,v 1.2 2005/04/26 23:45:43 rh Exp $
 *
 * a simple, incomplete ubiquitous stack data type
 *
 */

#include <stdlib.h>
#include <math.h>
#include "stack.h"

#define stack_ResizeIfNeeded(s) ((s)->n < (s)->capacity ?		\
					(s) :				\
					stack_Resize((s), 2*(s)->capacity))

#ifndef NAN
#define NAN	(0.0/0.0)
#endif

/*
 * create a new stack
 */
Stack *new_Stack(void)
{
	return stack_Init(stack_Alloc(STACK_DEFAULT_CAPACITY));
}


/*
 * allocate a new stack with a given capcity
 *
 * parameters:
 *	- capacity: number of elements this stack can take ( > 0 )
 *
 * return values:
 *	a valid stack if successful, NULL otherwise
 */
Stack *stack_Alloc(int cap)
{
	Stack *this = malloc(sizeof(*this));

	if (this == NULL) return NULL;

	this->value = malloc(cap * sizeof(StackValue));

	if (this->value == NULL)
	{
		stack_Release(this);
		return NULL;
	}

	this->capacity = cap;

	return this;
}


/*
 * Resize the given stack
 *
 * parameters:
 *	- this: the stack to resize
 *	- newsize: the new size
 *
 * return values:
 *	the passed stack if successful, NULL otherwise
 */
Stack *stack_Resize(Stack *this, int newsize)
{
	StackValue *newval = realloc(this->value, newsize * sizeof(StackValue));

	if (newval == NULL) return NULL;

	this->value = newval;

	return this;
}


/*
 * initialise (empty) the current stack
 *
 * parameters:
 *	- this: the stack to initialise
 *
 * return values:
 *	the passed stack
 */
Stack *stack_Init(Stack *this)
{
	this->n = 0;

	return this;
}


/*
 * release the given stack
 *
 * parameters:
 *	- this: the stack to release
 *
 * return values:
 *	none
 */
void stack_Release(Stack *this)
{
	if (this->value) free(this->value);
	free(this);
}


/*
 * get the number of elements on the stack
 *
 * parameters:
 *	- this: the stack to inspect
 *
 * return values:
 *	the number of elements on the stack
 */
int stack_numberOfElements(Stack *this)
{
	return this->n;
}


/*
 * push a double value onto the stack
 *
 * parameters:
 *	- this: the stack to manipulate
 *	- value: the value to push
 *
 * return values
 *	the passed stack if successful, NULL otherwise
 */
Stack *stack_pushDouble(Stack *this, double value)
{
	if (!stack_ResizeIfNeeded(this))
		return NULL;		/* out of memory? */

	this->value[this->n++].double_value = value;

	return this;
}


/*
 * pop a double value off a non-empty stack
 *
 * parameters:
 *	- this: the stack to get the value from
 *
 * return values:
 *	the double value
 */
double stack_popDouble(Stack *this)
{
	return this->n > 0 ? this->value[--this->n].double_value : NAN;
}

